using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Data.SqlServerCe;
using System.IO;

namespace CEQuery
{
    public partial class OpenDB : Form
    {
        public OpenDB()
        {
            InitializeComponent();
        }
        public OpenDB(string startupFile)
        {
            InitializeComponent();

            if (System.IO.File.Exists(startupFile))
            {
                CEFile = startupFile;
                txtFile.Text = startupFile;
                txtPassword.Focus();
            }
        }
        public string CEFile { get; private set; }
        public string ConnectionString { get; private set; }
        private bool ValidateInput()
        {
            bool isValid = true;
            if (this.txtFile.Text.Trim() == string.Empty)
            {
                isValid = false;
                errValidation.SetError(btnSelect, "Select a file.");
            }
            else
            {
                errValidation.SetError(btnSelect, string.Empty);
            }
            return isValid;
        }
        private void btnSelect_Click(object sender, EventArgs e)
        {
            openFileDialogue.Title = "Open the SQL CE File";
            openFileDialogue.Filter = "SQL CE Files (*.sdf)|*.sdf";
            openFileDialogue.RestoreDirectory = true;
            if (openFileDialogue.ShowDialog() == DialogResult.OK)
            {
               txtFile.Text = openFileDialogue.FileName;
               txtPassword.Focus();
            }
        }

        private void OpenDB_Load(object sender, EventArgs e)
        {

        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            if (!ValidateInput())
            {
                return;
            }
            //ConnectionString = string.Format("Data Source={0};Password={1};Max Database Size=4091;Max Buffer Size = 1024;Default Lock Escalation =100;Encrypt Database=TRUE", txtFile.Text, txtPassword.Text);
            CEFile = txtFile.Text;
            ConnectionString = string.Format("Data Source={0};Password={1};Max Database Size=4091;Max Buffer Size = 1024;Default Lock Escalation =100;Encrypt Database={2}", txtFile.Text, txtPassword.Text, txtPassword.Text.Length == 0 ? "FALSE" : "TRUE");
            using (SqlCeConnection connection = new SqlCeConnection(ConnectionString))
            {
                try
                {
                    connection.Open();
                    btnHiddenOK.PerformClick();
                }
                catch (SqlCeInvalidDatabaseFormatException)
                {
                    if (MessageBox.Show(this, "This DB has been created in older version of SQL CE and needs to be upgraded. Do you want to upgrade it now?", "CEQuery", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == System.Windows.Forms.DialogResult.Yes)
                    {
                        try
                        {
                            File.Copy(txtFile.Text, txtFile.Text.Substring(0, txtFile.Text.LastIndexOf('.')) + "_Backup.sdf");
                            using (SqlCeEngine engine = new SqlCeEngine(ConnectionString))
                            {
                                engine.Upgrade();
                                MessageBox.Show("Upgradation has been done sucessfully. A backup has been kept in the same location. Try to reload the file again.");
                            }
                        }
                        catch(Exception innerEx)
                        {
                            MessageBox.Show(innerEx.Message);
                        }
                    }
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }                
            }
        }

    }
}